@@ -182,6 +182,13 @@ class Compiler(object):
                 raise
 
     def compile(self, *filenames):
+        # TODO this doesn't spit out the compilation itself, so if you want to
+        # get something out besides just the output, you have to copy this
+        # method.  that sucks.
+        # TODO i think the right thing is to get all the constructors out of
+        # SourceFile, since it's really the compiler that knows the import
+        # paths and should be consulted about this.  reconsider all this (but
+        # preserve it for now, SIGH) once importers are a thing
         compilation = self.make_compilation()
         for filename in filenames:
             # TODO maybe SourceFile should not be exposed to the end user, and
@@ -274,41 +281,29 @@ class Compilation(object):
         return source
 
     def run(self):
-        # this will compile and manage rule: child objects inside of a node
-        self.parse_children()
+        # Any @import will add the source file to self.sources and infect this
+        # list, so make a quick copy to insulate against that
+        # TODO maybe @import should just not do that?
+        for source_file in list(self.sources):
+            rule = SassRule(
+                source_file=source_file,
+                lineno=1,
 
-        # this will manage @extends
+                unparsed_contents=source_file.contents,
+                namespace=self.root_namespace,
+            )
+            self.rules.append(rule)
+            self.manage_children(rule, scope=None)
+            self._warn_unused_imports(rule)
+
+        # Run through all the rules and apply @extends in a separate pass
         self.rules = self.apply_extends(self.rules)
 
-        rules_by_file, css_files = self.parse_properties()
+        output, total_selectors = self.create_css(self.rules)
+        if total_selectors >= 4096:
+            log.error("Maximum number of supported selectors in Internet Explorer (4095) exceeded!")
 
-        all_rules = 0
-        all_selectors = 0
-        exceeded = ''
-        final_cont = ''
-        files = len(css_files)
-        for source_file in css_files:
-            rules = rules_by_file[source_file]
-            fcont, total_rules, total_selectors = self.create_css(rules)
-            all_rules += total_rules
-            all_selectors += total_selectors
-            # TODO i would love for the output of this function to be something
-            # useful for producing stats, so this stuff can live on the Scss
-            # class only
-            if not exceeded and all_selectors > 4095:
-                exceeded = " (IE exceeded!)"
-                log.error("Maximum number of supported selectors in Internet Explorer (4095) exceeded!")
-            if files > 1 and self.compiler.generate_source_map:
-                final_cont += "/* %s %s generated from '%s' add up to a total of %s %s accumulated%s */\n" % (
-                    total_selectors,
-                    'selector' if total_selectors == 1 else 'selectors',
-                    source_file.path,
-                    all_selectors,
-                    'selector' if all_selectors == 1 else 'selectors',
-                    exceeded)
-            final_cont += fcont
-
-        return final_cont
+        return output
 
     def parse_selectors(self, raw_selectors):
         """
@@ -333,26 +328,6 @@ class Compilation(object):
 
         return selectors, parents
 
-    # @print_timing(3)
-    def parse_children(self, scope=None):
-        children = []
-        root_namespace = self.root_namespace
-        for source_file in self.sources:
-            rule = SassRule(
-                source_file=source_file,
-                lineno=1,
-
-                unparsed_contents=source_file.contents,
-                namespace=root_namespace,
-            )
-            self.rules.append(rule)
-            children.append(rule)
-
-        for rule in children:
-            self.manage_children(rule, scope)
-
-        self._warn_unused_imports(self.rules[0])
-
     def _warn_unused_imports(self, rule):
         if not rule.legacy_compiler_options.get(
                 'warn_unused', self.compiler.warn_unused_imports):
@@ -709,6 +684,7 @@ class Compilation(object):
                         return e.retval
                     else:
                         return Null()
+                __call._pyscss_needs_namespace = True
                 return __call
             _mixin = _call(mixin)
             _mixin.mixin = mixin
@@ -1043,10 +1019,8 @@ class Compilation(object):
         _rule = rule.copy()
         _rule.unparsed_contents = block.unparsed_contents
         _rule.namespace = rule.namespace
-        _rule.properties = {}
+        _rule.properties = []
         self.manage_children(_rule, scope)
-        for name, value in _rule.properties.items():
-            rule.namespace.set_variable(name, value)
     _at_vars = _at_variables
 
     # @print_timing(10)
@@ -1063,7 +1037,7 @@ class Compilation(object):
         except IndexError:
             is_var = False
         if is_var:
-            warn_deprecated(rule, "Assignment with = is deprecated; use :.")
+            warn_deprecated(rule, "Assignment with = is deprecated; use : instead.")
         calculator = self._make_calculator(rule.namespace)
         prop = prop.strip()
         prop = calculator.do_glob_math(prop)
@@ -1121,6 +1095,8 @@ class Compilation(object):
                 # TODO kill this branch
                 pass
             else:
+                if value.is_null:
+                    return
                 style = rule.legacy_compiler_options.get(
                     'style', self.compiler.output_style)
                 compress = style == 'compressed'
@@ -1268,7 +1244,7 @@ class Compilation(object):
                     # TODO implement !optional
                     warn_deprecated(
                         rule,
-                        "Can't find any matching rules to extend {0!r} -- this"
+                        "Can't find any matching rules to extend {0!r} -- this "
                         "will be fatal in 2.0, unless !optional is specified!"
                         .format(selector.render()))
                     continue
@@ -1315,25 +1291,6 @@ class Compilation(object):
         # Remove placeholder-only rules
         return [rule for rule in rules if not rule.is_pure_placeholder]
 
-    # @print_timing(3)
-    def parse_properties(self):
-        css_files = []
-        seen_files = set()
-        rules_by_file = {}
-
-        for rule in self.rules:
-            source_file = rule.source_file
-            rules_by_file.setdefault(source_file, []).append(rule)
-
-            if rule.is_empty:
-                continue
-
-            if source_file not in seen_files:
-                seen_files.add(source_file)
-                css_files.append(source_file)
-
-        return rules_by_file, css_files
-
     # @print_timing(3)
     def create_css(self, rules):
         """
@@ -1407,7 +1364,6 @@ class Compilation(object):
 
         prev_ancestry_headers = []
 
-        total_rules = 0
         total_selectors = 0
 
         result = ''
@@ -1488,7 +1444,6 @@ class Compilation(object):
                     header_string = header.render()
                 result += tb * (i + nesting) + header_string + sp + '{' + nl
 
-                total_rules += 1
                 if header.is_selector:
                     total_selectors += 1
 
@@ -1507,7 +1462,7 @@ class Compilation(object):
         if not result.endswith('\n'):
             result += '\n'
 
-        return (result, total_rules, total_selectors)
+        return (result, total_selectors)
 
     def _print_properties(self, properties, sc=True, sp=' ', tb='', nl='\n', lnl=' '):
         result = ''
